#!/usr/bin/env node

const httpServerManager = require('../src/http/HttpServerManager');
const requestUtils = require('../src/utils/RequestUtils');
const consoleUtils = require('../src/utils/ConsoleUtils');
const path = require('path');
const metadata = require('../metadata.json');

let startTime = Date.now();
let pids = metadata.pids;
if (pids) {
    pids.forEach(pid => {
        try {
            process.kill(pid);
        } catch (e) {}
    });
}
metadata.pids = [];//服务启动成功后，会保存metadata信息到磁盘

let promise = httpServerManager.start();
promise.then(() => {
    let httpStarted = Date.now();
    console.log(`http服务启动完成，耗时【${httpStarted-startTime}】毫秒！`);
    return requestUtils.request({
        commander: "webpack",
        componentSrc: path.resolve(__dirname, '../components/_default')
    });
}).then(data => {
    if (data.success) {
        let duringTime = Date.now() - startTime;
        consoleUtils.out({
            success: true,
            type: "ServerStart",
            title: "服务启动成功！",
            content: `服务启动成功，耗时【${duringTime}】毫秒！`
        });
    } else {
        consoleUtils.out(data);
    }
}).catch(err => {
    consoleUtils.out({
        success: false,
        type: "ServerStartException",
        title: "服务启动失败！",
        content: typeof (err) == "string" ? err : (err.stack ? err.stack : err.message)
    });
});