const ExtractTextPlugin = require('extract-text-webpack-plugin');
const postcss = require('postcss');
const postCssUrl = require('postcss-url');
const autoprefixer = require('autoprefixer');
const path = require('path');
const fs = require('fs');
const webpack = require('webpack');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const webpackUtils = require('./webpack.v3.utils');
const postCssUrlCfg = webpackUtils.postCssUrlCfg;
let metadataPath = "../../../../../components/_default/src/components/metadata.json";

module.exports = {
    "entry": {
        "index": "./build/v3/preview/index.js",
        "itop/resources/_default_index.lib": "./src/components/_default/views/index/index.js",
        "_default_index_index_previewIndex": "./src/components/_default/views/index/test/index_previewIndex.js"
    },
    "output": {
        "path": path.resolve(__dirname, '../../', 'dist'),
        "filename": "[name].js",
        "libraryTarget": "umd",
        "umdNamedDefine": true
    },
    "context":path.resolve(__dirname,'../../'),
    "node":{
        fs : "empty"
    },
    "resolve": {
        "extensions": [".js", ".vue"],
        "alias": {
            "vue": "vue/dist/vue.esm.js",
            "@components": path.join(__dirname, '../../', 'src/components'),
            "@vPlatform": webpackUtils.toPluginPath("v3webpack-cli", "src/vendor/vplatform"),
            "@_default": path.resolve(__dirname, '../../', 'src/components/_default')
        }
    },
    "performance": {
        "hints": false
    },
    "externals": webpackUtils.getExternals(),
    "module": {
        "rules": [{
            "test": /.css$/,
            "use": ExtractTextPlugin.extract({
                fallback: "vue-style-loader",
                use: [
                    "css-loader",
                    {
                        loader: 'postcss-loader',
                        options: {
                            plugins: [
                                autoprefixer(),
                                postCssUrl(postCssUrlCfg)
                            ]
                        }
                    }
                ]
            })
        }, {
            "test": /.vue$/,
            "loader": "vue-loader",
            "options": {
                "loaders": {
                    "css": ExtractTextPlugin.extract({
                        use: 'css-loader',
                        fallback: 'vue-style-loader'
                    }),
                    "less": ExtractTextPlugin.extract({
                        use: 'css-loader!less-loader',
                        fallback: 'vue-style-loader'
                    })
                },
                "postLoaders": {
                    "html": "babel-loader"
                },
                "postcss": {
                    "plugins": [autoprefixer(), postCssUrl(postCssUrlCfg)]
                }
            }
        }, {
            "test": /.js$/,
            "loader": "babel-loader",
            "exclude": /node_modules/
        }, {
            "test": /.ejs$/,
            "loader": "ejs-loader?metadataPath="+metadataPath
        }, {
            "test": /.(png|jpe?g|gif|svg|woff|woff2|eot|ttf)(\?\S*)?$/,
            "loader": "v3-file-loader",
            "options": {
                "name": "itop/resources/_default_[sha512:hash:base64:7].[ext]",
                "publicPath": function(url) {
                    let queryStr = this.resourceQuery;
                    if (queryStr && this.resourceQuery.indexOf("cssRes=true") != -1) {
                        let array = url.split("/");
                        return "./" + array.pop();
                    } else {
                        return url;
                    }
                }
            }
        }]
    },
    "plugins": [
        new webpack.HashedModuleIdsPlugin(),
        new CopyWebpackPlugin([{
            from: webpackUtils.toPluginPath('v3webpack-cli', 'src/vendor/vplatform/assets'),
            to: path.join(__dirname, '../../', 'dist/vendor/vplatform/assets')
        }]),
        new webpack.optimize.CommonsChunkPlugin({
            name: "vendor",
            filename: "itop/resources/_default_vendor.lib.js",
        }),
        new HtmlWebpackPlugin({
            filename: 'index.html',
            template: webpackUtils.toPluginPath('v3webpack-cli', 'src/vendor/vplatform/index.html'),
            chunks: ["vendor", "index"]
        }),
        new ExtractTextPlugin({
            filename: "[name].css"
        }),
        new HtmlWebpackPlugin({
            filename: "_default_index_index.html",
            template: webpackUtils.toPluginPath('v3webpack-cli', 'src/vendor/vplatform/windowPreview.html'),
            chunks: ["vendor", "itop/resources/_default_index.lib", "_default_index_index_previewIndex"]
        })
    ]
};

if (process.env.NODE_ENV === 'production') {
    module.exports.plugins.push(
        new webpack.DefinePlugin({
            'process.env': {
                NODE_ENV: '"production"'
            }
        }),
        new webpack.optimize.UglifyJsPlugin({
            sourceMap: true,
            compress: {
                warnings: false
            }
        }),
        new webpack.LoaderOptionsPlugin({
            minimize: true
        }));
} else {
    module.exports.devServer = {
        open: true,
        inline: true,
        host: "localhost",
        port: 9090,
        contentBase: "./dist/"
    };

    module.exports.devtool = '#eval-source-map';
}