const Command = require('../Command');
const CommandResult = require('../CommandResult');
const CommandContextKey = require('../CommandContextKey');
const fileUtils = require('../../../utils/FileUtils');
const path = require('path');
const ioUtils = require('../../../utils/IOUtils');
const fs = require('fs');

/**
 * 移除构件
 */
class DeleteCommand extends Command {

    deleteCache(filePath){
        if(Array.isArray(filePath)){
            filePath.forEach(p=>{
                this.deleteCache(p);
            });
        }else{
            delete require.cache[filePath];
        }
    }

    invoke(params, context) {
        return new Promise((resolve, reject) => {
            let componentCode = params.componentCode;
            let componentDir = path.resolve(__dirname, '../../../../components', componentCode);
            if (fs.existsSync(componentDir)) {
                let promise = fileUtils.delete(componentDir);
                promise.then((deletedFiles) => {
                    return new Promise((rsl, rej) => {
                        try {
                            if (deletedFiles) {
                                this.deleteCache(deletedFiles);
                            }
                            let metadata = require('../../../../metadata.json');
                            let filemetadata = metadata.filemetadata;
                            if (filemetadata) {
                                let compare = componentCode + path.sep;
                                let deleted = false;
                                for (let path of Object.keys(filemetadata)) {
                                    if (path.startsWith(compare)) {
                                        delete filemetadata[path];
                                        deleted = true;
                                    }
                                }
                                if (deleted) {
                                    let promise = ioUtils.write(path.resolve(__dirname, '../../../../metadata.json'), JSON.stringify(metadata, null, "\t"));
                                    promise.then(() => {
                                        rsl();
                                    }).catch(err => {
                                        rej(err);
                                    });
                                } else {
                                    rsl();
                                }
                            } else {
                                rsl();
                            }
                        } catch (e) {
                            rej(e);
                        }
                    });
                }).then(() => {
                    resolve(new CommandResult({
                        success: true,
                        type: "ComponentDeleted",
                        title: "构件移除成功",
                        content: ''
                    }));
                }).catch(err => {
                    reject(err);
                });
            } else {
                resolve(new CommandResult({
                    success: true,
                    type: "ComponentDeleted",
                    title: "构件移除成功",
                    content: ''
                }));
            }
        });
    }

}

DeleteCommand.getCommand = function () {
    return "delete";
};

module.exports = DeleteCommand;