const Command = require('../Command');
const ComponentUpdater = require('../../../updater/ComponentUpdater');
const NodejsPluginUpdater = require('../../../updater/NodejsPluginUpdater');
const ComponentWebpack = require('../../../webpack/ComponentWebpack');
const CommandResult = require('../CommandResult');
const path = require('path');
const selfDir = path.resolve(__dirname,'../../../../components');

class WebpackCommand extends Command {

    constructor(){
        super();
        this.currentComponent = null;
    }

    getWebpacker(created = true) {
        if (!this.webpacker && created) {
            this.webpacker = new ComponentWebpack();
        }
        return this.webpacker;
    }

    invoke(params, context) {
        return new Promise((resolve, reject) => {
            let startTime = Date.now();
            let src = params.componentSrc,
                promises = [];
            let nopreview = params.nopreview;
            if(nopreview=="true"){
                process.env.NODE_ENV = "production";
            }else{
                process.env.NODE_ENV = "develepment";
            }
            if(!src.startsWith(selfDir)){
                let nodejsPluginUpdater = new NodejsPluginUpdater(src);
                let promise = nodejsPluginUpdater.update();
                promises.push(promise);
                let componentUpdater = new ComponentUpdater(src);
                promise = componentUpdater.udpate();
                promises.push(promise);
            }
            Promise.all(promises).then(() => {
                let endTime = Date.now();
                let webpacker = this.getWebpacker();
                return webpacker.pack(src);
            }).then((assets) => {
                resolve(new CommandResult({
                    success: true,
                    type: "ComponentWebpack",
                    title: "构件打包成功",
                    content: assets
                }));
            }).catch(err => {
                reject(err);
            });
        });
    }

    beforeClose() {
        let webpacker = this.getWebpacker(false);
        if (webpacker) {
            return webpacker.close();
        } else {
            return new Promise((resolve, reject) => {
                resolve();
            });
        }
    }

}

WebpackCommand.getCommand = function () {
    return "webpack";
};

module.exports = WebpackCommand;