const fileUtils = require('../utils/FileUtils');
const ioUtils = require('../utils/IOUtils');
const path = require('path');
const juicer = require('juicer');
const fs = require('fs');
const beautify = require('js-beautify');

let formatMapping = {
    babelrc: "json"
};

class ComponentTemplateGenerator {

    constructor(dist, params) {
        this.dist = dist;
        this.params = params || {};
    }

    getTemplateDir() {
        return path.resolve(__dirname, 'tpls');
    }

    toSavePath(templatePath) {
        let tmpDir = this.getTemplateDir();
        let relativePath = path.relative(tmpDir, templatePath);
        if (relativePath.endsWith(".tpl")) {
            relativePath = relativePath.substring(0, relativePath.length - 4);
        }
        return path.resolve(this.dist, relativePath);
    }

    createFile(tmpPath, savePath) {
        return new Promise((resolve, reject) => {
            fs.readFile(tmpPath, (err, content) => {
                if (err) {
                    return reject(err);
                }
                try {
                    juicer.set({
                        'strip': false,
                        'cache': false
                    });
                    content = juicer(String(content), this.params);
                    let subfix = this.getSubfix(savePath);
                    content = this.formatResource(this.toFormatType(subfix), content);
                    let promise = ioUtils.write(savePath, content);
                    promise.then(() => {
                        resolve();
                    }).catch((e) => {
                        reject(e);
                    });
                } catch (err) {
                    reject(err);
                }
            });
        });
    }

    getSubfix(path) {
        return path.substring(path.lastIndexOf('.') + 1);
    }

    toFormatType(subfix) {
        return formatMapping[subfix] ? formatMapping[subfix] : subfix;
    }

    formatResource(type, content) {
        switch (type) {
            case 'js':
                content = beautify.js(content);
                break;
            case 'html':
                content = beautify.html(content);
                break;
            case 'css':
            case 'less':
                content = beautify.css(content);
                break;
            case 'json':
                let jsonObj = JSON.parse(content);
                content = JSON.stringify(jsonObj, null, "\t");
                break;
        }
        return content;
    }

    generate() {
        return new Promise((resolve, reject) => {
            let templateDir = this.getTemplateDir();
            let files = fileUtils.listFiles(templateDir,true);
            let promises = [];
            files.forEach(file => {
                let savePath = this.toSavePath(file);
                promises.push(this.createFile(file, savePath));
            });
            Promise.all(promises).then(() => {
                resolve();
            }).catch(err => {
                reject(err);
            });
        });
    }


}

module.exports = ComponentTemplateGenerator;