const fs = require('fs');
const path = require('path');
const pathUtils = require('./PathUtils');

module.exports = {

	makeDir: function (path) {
		let iter = (parent) => {
			if (!this.exists(parent)) {
				let f = pathUtils.getParent(parent);
				iter(f);
				fs.mkdirSync(parent);
			}
		};
		let parent = pathUtils.getParent(path);
		iter(parent);
		if (!this.exists(path)) {
			fs.mkdirSync(path);
		}
	},

	exists: function (path) {
		return fs.existsSync(path);
	},

	isFile: function (path) {
		if (fs.existsSync(path)) {
			let stat = fs.statSync(path);
			return stat.isFile();
		}
		return false;
	},

	isDirectory: function (path) {
		if (fs.existsSync(path)) {
			let stat = fs.statSync(path);
			return stat.isDirectory();
		}
		return false;
	},

	listFiles: function (filePath, recursion) {
		let files = [];
		if (fs.existsSync(filePath)) {
			let stat = fs.statSync(filePath);
			if (stat.isFile()) {
				files.push(filePath);
			} else if (stat.isDirectory() && recursion) {
				let paths = fs.readdirSync(filePath);
				for (let i = 0, len = paths.length; i < len; i++) {
					files = files.concat(this.listFiles(path.resolve(filePath, paths[i]), recursion));
				}
			}
		}
		return files;
	},

	delete: function (filePath) {
		return new Promise((resolve, reject) => {
			try {
				if (this.isFile(filePath)) {
					fs.unlink(filePath, (err) => {
						if (err) {
							return reject(err);
						}
						resolve(filePath);
					});
				} else {
					fs.readdir(filePath, (err, fileNames) => {
						if (err) {
							return reject(err);
						}
						if (fileNames && fileNames.length > 0) {
							let promises = [];
							fileNames.forEach(fileName => {
								promise = this.delete(path.resolve(filePath, fileName));
								promises.push(promise);
							});
							Promise.all(promises).then((args) => {
								fs.rmdir(filePath,(err)=>{
									if(err){
										return reject(err);
									}
									resolve(args);
								});
							}).catch(err => {
								reject(err);
							});
						} else {
							fs.rmdir(filePath,(err)=>{
								if(err){
									return reject(err);
								}
								resolve();
							});
						}
					});
				}
			} catch (err) {
				reject(err);
			}
		});
	}

};